
#include <windows.h>  
#include <stdio.h>
#include "setpos.h"
#include "sqlext.h"  


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "SQLSetPos()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

#define NAME_LEN 20

static int   nDeptId;
static UCHAR sDeptName[NAME_LEN+1];
static int   nDeptHeadId;

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static HENV  hEnv  = NULL;
static HDBC  hDBC  = NULL;
static HWND  hList = NULL;

static UCHAR szLBStr[256];

UCHAR  szSqlStr[] = "Select dept_id, dept_name, dept_head_id From department";

   switch( uMsg )
   {
      case WM_CREATE :
              {
                 RETCODE retCode;

                 // Allocate Environment and Connection.
                 //.....................................
                 SQLAllocEnv( &hEnv );
                 SQLAllocConnect( hEnv, &hDBC );

                 // Connect to the data source
                 //...........................
                 retCode = SQLConnect( hDBC, 
                             "Test Data Source", SQL_NTS,
                             "DBA",              SQL_NTS,  
                             "SQL",              SQL_NTS );

                 if ( retCode != SQL_SUCCESS )
                 {
                    SQLFreeConnect( hDBC );
                    SQLFreeEnv( hEnv );
                    return( -1 );
                 }

                 if ( retCode == SQL_SUCCESS )
                    hList = CreateWindow( "LISTBOX", "",    
                                          LBS_NOTIFY | WS_VSCROLL | 
                                          WS_BORDER  | WS_CHILD | 
                                          WS_VISIBLE | LBS_NOINTEGRALHEIGHT, 
                                          0, 0, 
                                          0, 0,  
                                          hWnd,              
                                          (HMENU)101,              
                                          hInst,         
                                          NULL               
                                        );
              }
              break;

      case WM_SIZE :
              MoveWindow( hList, 0, 0, LOWORD( lParam ), 
                                       HIWORD( lParam ), TRUE );
              break; 
              
      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_TEST :
                     {
                        HSTMT   hStmt;
                        RETCODE retCode;
                        SDWORD  cbValue1 = sizeof(int);
                        SDWORD  cbValue2 = NAME_LEN;
                        SDWORD  cbValue3 = sizeof(int);
                        UDWORD  crow;
                        UWORD   rgfRowStatus;
                        SDWORD  nRowsAdded;
                        
                        // Allocate a statement handle for the query.
                        // ..........................................
                        SQLAllocStmt( hDBC, &hStmt );
                        SQLSetStmtOption( hStmt, SQL_CONCURRENCY, SQL_CONCUR_LOCK );

                        // Call SQLExecDirect() to compile
                        // and execute the query.
                        // ...............................
                        retCode = SQLExecDirect( hStmt, szSqlStr, SQL_NTS );
                        
                        if( retCode == SQL_SUCCESS ||
                            retCode == SQL_SUCCESS_WITH_INFO )
                        {
                           // Bind columns.
                           // .............
                           SQLBindCol( hStmt, 1, SQL_C_SLONG, &nDeptId, 
                                       0, &cbValue1 );
                           SQLBindCol( hStmt, 2, SQL_C_CHAR,  sDeptName, 
                                       NAME_LEN, &cbValue2 );
                           SQLBindCol( hStmt, 3, SQL_C_SLONG, &nDeptHeadId, 
                                       0, &cbValue3 );

                           // Fetch the records.
                           //...................
                           retCode = SQLExtendedFetch( hStmt, 
                              SQL_FETCH_NEXT, 0, &crow, &rgfRowStatus );

                           if ( retCode == SQL_SUCCESS || 
                                retCode == SQL_NO_DATA_FOUND ||
                                retCode == SQL_SUCCESS_WITH_INFO )
                           {
                              // assign data
                              //............

                              cbValue1 = sizeof(int);
                              cbValue2 = NAME_LEN;   
                              cbValue3 = sizeof(int);

                              nDeptId = 600;
                              strcpy(sDeptName, "New Business");
                              nDeptHeadId = 902;

                              // Perform an add on the department table.
                              // .......................................
                              retCode = SQLSetPos( hStmt, 1, SQL_ADD,
                                                   SQL_LOCK_NO_CHANGE );

                              if( retCode == SQL_SUCCESS )
                              {
                                 SQLTransact( SQL_NULL_HENV, hDBC, SQL_COMMIT );

                                 // Determine the number of rows 
                                 // that were updated.
                                 // ............................
                                 SQLRowCount( hStmt, &nRowsAdded );

                                 // Report the number of updated 
                                 // columns to the user.
                                 // ............................
                                 szLBStr[0] = '\0';
                                 sprintf( szLBStr, 
                                   "Number of rows added = %ld", nRowsAdded );

                                 SendMessage( hList, LB_ADDSTRING, 
                                              0, (LPARAM)szLBStr );
                              }
                              else
                              {
                                 SendMessage( hList, LB_ADDSTRING, 0, 
                                              (LPARAM)"Update Failed" );

                              }
                           }
                        }

                        // Free the statement handle.
                        // ..........................
                        SQLFreeStmt( hStmt, SQL_DROP );
                     }
                     break;

                 case IDM_ABOUT :
                     DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                     break;

                 case IDM_EXIT :
                     DestroyWindow( hWnd );
                     break;
              }
              break;
      
      case WM_DESTROY :
              PostQuitMessage(0);

              // Disconnect Environment.
              //........................
              SQLDisconnect( hDBC );

              // Free Connection and Environment.
              //.................................
              SQLFreeConnect( hDBC );
              SQLFreeEnv( hEnv );
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
